from pandas import DataFrame
from .request import get_json

metadata_list = ["codigo_sgs", "title", "periodicidade", "unidade_medida"]


def bcb_get_search_results(url):
    """
    Auxiliary function to request BCB's search
    results and turn json into a DataFrame.

    Parameters
    ----------
    url : str
        Url to be requested.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with the search results.

    Raises
    ------
    AssertionError
        If nothing was found.
    """
    json = get_json(url)
    search_results = json["result"]["results"]
    assert search_results, "Nothing was found."
    return DataFrame(search_results).loc[:, metadata_list]


def ipea_get_search_results(url):
    """
    Auxiliary function to request IPEA's search
    results and turn json into a DataFrame.

    Parameters
    ----------
    url : str
        Url to be requested.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with the search results.

    Raises
    ------
    AssertionError
        If nothing was found.
    """
    json = get_json(url)
    search_results = json["value"]
    assert search_results, "Nothing was found."
    return DataFrame(search_results)



import pandas as pd
from .request import get_json


def bcb_json_to_df(url, code, name):
    """
    Auxiliary function to convert json produced by
    BCB's and IPEA's API into a DataFrame.

    Parameters
    ----------
    url : str
        Url to be requested.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with time series' values and a DateTimeIndex.
    """
    json = get_json(url)
    df = pd.DataFrame(json)
    df["data"] = pd.to_datetime(df["data"], format="%d/%m/%Y")
    df["valor"] = df["valor"].astype('float64')
    df = df.set_index("data")
    df = df.rename_axis("Date")
    df.columns = [name if name else code]
    return df


def ipea_json_to_df(url, code, name):
    """
    Auxiliary function to convert json produced by
    BCB's and IPEA's API into a DataFrame.

    Parameters
    ----------
    url : str
        Url to be requested.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with time series' values and a DateTimeIndex.
    """
    json = get_json(url)["value"]
    df = pd.DataFrame(json)
    df["VALDATA"] = df["VALDATA"].str[:-6]
    df["VALDATA"] = pd.to_datetime(df["VALDATA"], format="%Y-%m-%dT%H:%M:%S")
    df["VALVALOR"] = df["VALVALOR"].astype('float64')
    df = df.set_index("VALDATA")
    df = df.rename_axis("Date")
    df.columns = [name if name else code]
    return df


def ibge_json_to_df(url, freq="mensal"):
    """
    Auxiliary function to convert json produced by
    IBGE's API into a DataFrame.

    Parameters
    ----------
    url : str
        Url to be requested.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with time series' values, metadatas
        and a DateTimeIndex.
    """
    json = get_json(url)
    df = pd.DataFrame(json[1:])
    df.columns = json[0].values()
    date_fmt = "%Y" if freq == "anual" else "%Y%m"
    date_key = json[0]["D2C"]
    df[date_key] = pd.to_datetime(df[date_key], format=date_fmt)
    df["Valor"] = pd.to_numeric(df["Valor"], errors="coerce")
    df = df.set_index(date_key)
    df = df.rename_axis("Date")
    df = df.drop(
        [c for c in df.columns if c.endswith("(Código)")]
        + ["Mês", "Unidade de Medida", "Brasil"],
        axis="columns",
        errors="ignore",
    )
    return df



import re
from pandas import concat
from . import bcb, ipea
from .helpers.utils import return_codes_and_names


def get_series(*codes, start=None, end=None, **kwargs):
    """
    Get multiple series from both BCB or IPEA.

    Parameters
    ----------
    codes : dict, str, int
        Dictionary like {"name1": cod1, "name2": cod2}
        or a bunch of code numbers, e.g. cod1, cod2.

    start : str, optional
        Initial date, month or day first.

    end : str, optional
        End date, month or day first.

    last_n : int, optional
        Ignore other arguments and get last n observations.

    **kwargs
        Passed to pandas.concat.

    Returns
    -------
    pandas.DataFrame
        A DataFrame with series' values.

    Examples
    --------
    >>> seriesbr.get_series("BM12_CRLIN12", 20786)
                BM12_CRLIN12  20786
    Date
    2011-03-01          4.41  26.22
    2011-04-01          4.55  27.01
    2011-05-01          4.72  26.84
    2011-06-01          4.71  26.72
    2011-07-01          4.89  26.91
    ...                  ...    ...
    2019-06-01          3.83  31.43
    2019-07-01          3.96  31.63
    2019-08-01          3.88  31.57
    2019-09-01          3.89  30.84
    2019-10-01          3.88  30.35
    """
    codes, names = return_codes_and_names(*codes)
    series = []
    for code, name in zip(codes, names):
        if re.search(r"^\d+$", str(code)):
            series.append(bcb.get_serie(code, name, start, end))
        else:
            series.append(ipea.get_serie(code, name, start, end))
    return concat(series, axis="columns", **kwargs)



import re
import pandas as pd


def return_codes_and_names(*args):
    """
    Auxiliary function to label columns of a
    DataFrame.

    If it finds a dictionary, it will use its keys
    as labels and values as series' codes.

    Otherwise, the codes themselves will serve
    as columns' labels.

    Returns
    -------
    tuple
        First element with codes and second element
        with names.
    """
    for arg in args:
        if isinstance(arg, dict):
            return arg.values(), arg.keys()
    return args, args


def cat(something, sep):
    """
    Auxiliary function to join an iterable
    delimited by sep, forcing the join by
    coercing the items to be strings.

    Parameters
    ----------
    something : list
        List to be joined.

    sep : str
        Delimiter.

    Returns
    -------
    str
        String with values in something joined
        by sep.
    """
    return sep.join(map(str, something)) if isiterable(something) else something


def isiterable(something):
    """
    Auxiliary function to test if something is
    an iterable or not.

    Returns
    -------
    bool
        True if an iterable, False otherwise.
    """
    try:
        iter(something)
    except TypeError:
        return False
    return True


def do_search(df, search, searches):
    """
    Auxiliary function to search for regex
    in a column of a DataFrame.

    It builds a string to be passed to
    the query method of a DataFrame.

    Parameters
    ----------

    df : pandas.DataFrame

    search : list of strings

    searches : dict

    Raises
    ------
    ValueError
        If the user tries to search in a
        non-existent column.
    """
    df_cols = df.columns.tolist()
    for col in searches:
        if col not in df_cols:
            raise ValueError(f"{col} is a non-existing column.")
    regex = build_regex(search)
    name_search = f"nome.str.contains('{regex}')"
    if searches:
        other_searches = []
        for field, search in searches.items():
            regex = build_regex(search)
            other_searches.append(f"{field}.str.contains('{regex}')")
        other_searches = " and " + " and ".join(other_searches)
        return df.query(name_search + other_searches, engine='python')
    return df.query(name_search, engine='python')


def build_regex(strings):
    """
    Build regex by joining strings by '|'
    only if it is an iterable other than a str.
    """
    # (?iu) sets unicode and ignore case flags
    flags = r'(?iu)'
    if isinstance(strings, str):
        regex = f"{flags}{strings}"
    elif isiterable(strings):
        regex = f"{flags}{'|'.join(strings)}"
    return regex


def clean_json(json):
    """
    Helper function to turn JSON into a DataFrame
    and clean its columns names.
    """
    df = pd.io.json.json_normalize(json, sep='_')
    df = df.rename(lambda x: '_'.join(re.split(r'_', x)[-2:]), axis='columns')
    return df
